import pyodbc
from contextlib import contextmanager
from .create import create_html



def create_dataflow_package (server_name, database_name, connection_name_source, query, is_expression, package_qualifier, destination_connection,
                             destination_schema, destination_table, destination_truncate, keep_identity, use_bulk_copy, batch_size, command="regular"):
    if is_expression.strip() == '' or is_expression == 'is_expression(default 0)':
        is_expression = 0

    if destination_truncate.strip() == '' or destination_truncate == 'destination truncate default(0)':
        destination_truncate = 0

    if keep_identity.strip() == '' or keep_identity == 'keep identity default(0)':
        keep_identity = 0

    if use_bulk_copy.strip() == '' or use_bulk_copy == 'use bulk copy default(0)':
        use_bulk_copy = 0

    if batch_size.strip() == '' or batch_size == 'batch size default(0)':
        batch_size = 0


    @contextmanager
    def connection():
        try:
            cnxn = pyodbc.connect(
                "Driver={SQL Server Native Client 11.0};Server=" + server_name + ";Database=" + database_name + ";Trusted_Connection=yes;MARS_Connection=Yes;",
                autocommit=True)
            yield cnxn

        except Exception as e:
            print(e)

    val = 0

    try:
        with connection() as conn:
            curr = conn.cursor()
            params = (connection_name_source, query,  is_expression, package_qualifier, destination_connection,
                    destination_schema, destination_table,
                      destination_truncate, keep_identity, use_bulk_copy, batch_size)

            CreateProjectCommand = "SET NOCOUNT ON EXEC [elt].[Save Data Flow Package] ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?"
            curr.execute(CreateProjectCommand, params, )
            val = curr.fetchall()[0][0]
    except pyodbc.Error as e:
        print("Something got wrong")
        return

    if val:
        p_name = 'Data Flow ' + package_qualifier + ' ' + destination_table
        if command == "regular":
            print(f"Succesfully created dataflow pacakge {p_name}!")
        elif command == "clone":
            print(f"Succesfully cloned dataflow pacakge {p_name}!")

        create_html(server_name, database_name, p_name, command_type="data_flow")

def clone_dataflow_package (server_name, database_name, package_name, new_package_name):

    @contextmanager
    def connection():
        try:
            cnxn = pyodbc.connect(
                "Driver={SQL Server Native Client 11.0};Server=" + server_name + ";Database=" + database_name + ";Trusted_Connection=yes;MARS_Connection=Yes;",
                autocommit=True)
            yield cnxn

        except Exception as e:
            print(e)

    val = 0

    try:
        with connection() as conn:
            curr = conn.cursor()
            params = (package_name, new_package_name)

            CreateProjectCommand = "SET NOCOUNT ON EXEC [elt].[Clone Data Flow Package] ?, ?"
            curr.execute(CreateProjectCommand, params, )
            val = curr.fetchall()[0][0]
    except pyodbc.Error as e:
        print(e)

    if val > 0:
        print(f"Succesfully cloned project with qualifier {new_package_name} !")
        create_html(server_name, database_name, new_package_name, command_type="data_flow")


def delete_package(server_name, database_name, package_name):
    @contextmanager
    def connection():
        try:
            cnxn = pyodbc.connect(
                "Driver={SQL Server Native Client 11.0};Server=" + server_name + ";Database=" + database_name + ";Trusted_Connection=yes;MARS_Connection=Yes;",
                autocommit=True)
            yield cnxn

        except Exception as e:
            print(e)


    if package_name[:9] == "Data Flow":
        try:
            with connection() as conn:
                curr = conn.cursor()
                command = f"""DECLARE @name nvarchar(336) = '{package_name}'; DELETE FROM [elt].[package_config_data_flow] WHERE [package_name] = @name;
                         DELETE FROM [elt].[project_package] WHERE [package_name] = @name; 
                         DELETE FROM [elt].[query_history] WHERE [package_name] = @name;"""
                curr.execute(command)
                print(f"succesfully deleted the package {package_name}")
                create_html(server_name, database_name, package_name, command_type="data_flow")
        except Exception as e:
            print(e)

    if package_name[:15] == "Execute Process":
        try:
            with connection() as conn:
                curr = conn.cursor()
                command = f"""DECLARE @name nvarchar(128) = '{package_name}'; DELETE FROM [elt].[package_config_execute_process] WHERE [package_name] = @name; 
                DELETE FROM [elt].[dim_table_merge_config] WHERE [package_name] = @name; 
                DELETE FROM [elt].[package_config_execute_process_variable] WHERE [package_name] = @name; 
                DELETE FROM [elt].[project_package] WHERE [package_name] = @name; 
                DELETE FROM [elt].[query_history] WHERE [package_name] = @name;"""
                curr.execute(command)
                print(f"succesfully deleted the package {package_name}")
                create_html(server_name, database_name, package_name, command_type="execute_process")


        except Exception as e:
            print(e)

    if package_name[:11] == "Execute SQL":
        try:
            with connection() as conn:
                curr = conn.cursor()
                command = f"""DECLARE @name nvarchar(128) = '{package_name}'; DELETE FROM [elt].[package_config_execute_sql] WHERE [package_name] = @name; 
                DELETE FROM [elt].[fact_table_merge_config] WHERE [package_name] = @name; DELETE FROM [elt].[dim_table_merge_config] WHERE [package_name] = @name; 
                DELETE FROM [elt].[fact_table_partition_config] WHERE [package_name] = @name; DELETE FROM [elt].[fact_table_switch_config] WHERE [package_name] = @name;
                DELETE FROM [elt].[project_package] WHERE [package_name] = @name; DELETE FROM [elt].[query_history] WHERE [package_name] = @name;"""
                curr.execute(command)
                print(f"succesfully deleted the package {package_name}")
                create_html(server_name, database_name, package_name, command_type="execute_sql")

        except Exception as e:
            print(e)

    if package_name[:19] == "Foreach Execute SQL":
        try:
            with connection() as conn:
                curr = conn.cursor()
                command = f"""DECLARE @name nvarchar(336) = '{package_name}'; DELETE FROM [elt].[package_config_foreach_execute_sql] WHERE [package_name] = @name;
                 DELETE FROM [elt].[project_package] WHERE [package_name] = @name;
                 DELETE FROM [elt].[query_history] WHERE [package_name] = @name;"""
                curr.execute(command)
                print(f"succesfully deleted the package {package_name}")
                create_html(server_name, database_name, package_name, command_type="foreach_execute_sql")

        except Exception as e:
            print(e)

    if package_name[:17] == "Foreach Data Flow":
        try:
            with connection() as conn:
                curr = conn.cursor()
                command = f"""DECLARE @name nvarchar(336) = '{package_name}'; DELETE FROM [elt].[package_config_foreach_data_flow] WHERE [package_name] = @name; 
                DELETE FROM [elt].[project_package] WHERE [package_name] = @name; 
                DELETE FROM [elt].[query_history] WHERE [package_name] = @name;"""
                curr.execute(command)
                print(f"succesfully deleted the package {package_name}")
                create_html(server_name, database_name, package_name, command_type="foreach_data_flow")

        except Exception as e:
            print(e)




def create_foreach_dataflow_package(server_name, database_name, foreach_connection,connection_name_source, foreach_query,
    query, package_qualifier, destination_connection, destination_schema, destination_table, destination_truncate,
    keep_identity, use_bulk_copy, batch_size, command="regular"):

    if destination_truncate.strip() == '' or destination_truncate == 'destination truncate default(0)':
        destination_truncate = 0


    if keep_identity.strip() == '' or keep_identity == 'keep identity default(0)':
        keep_identity = 0

    if use_bulk_copy.strip() == '' or use_bulk_copy == 'use bulk copy default(0)':
        use_bulk_copy = 0

    if batch_size.strip() == '' or batch_size == 'batch size default(0)':
        batch_size = 0

    @contextmanager
    def connection():
        try:
            cnxn = pyodbc.connect(
                "Driver={SQL Server Native Client 11.0};Server=" + server_name + ";Database=" + database_name + ";Trusted_Connection=yes;MARS_Connection=Yes;",
                autocommit=True)
            yield cnxn
        except Exception as e:
            print(e)
    val = 0
    try:
        with connection() as conn:
            curr = conn.cursor()
            params = (foreach_connection,connection_name_source, foreach_query, query, package_qualifier, destination_connection,
                             destination_schema, destination_table, destination_truncate, keep_identity, use_bulk_copy, batch_size)

            CreateProjectCommand = "SET NOCOUNT ON EXEC [elt].[Save Foreach Data Flow Package] ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?"
            curr.execute(CreateProjectCommand, params, )
            val = curr.fetchall()[0][0]
    except pyodbc.Error as e:
        print("Something got wrong")
        return

    if val:
        p_name = 'Foreach Data Flow ' + package_qualifier + ' ' + destination_table
        if command == "regular":
            print(f"Succesfully created foreach dataflow pacakge {p_name}!")
        elif command == "clone":
            print(f"Succesfully cloned foreach dataflow pacakge {p_name}!")
        create_html(server_name, database_name, p_name, command_type="foreach_data_flow")


def clone_foreach_dataflow_package(server_name, database_name, package_name, new_package_name):
    @contextmanager
    def connection():
        try:
            cnxn = pyodbc.connect(
                "Driver={SQL Server Native Client 11.0};Server=" + server_name + ";Database=" + database_name + ";Trusted_Connection=yes;MARS_Connection=Yes;",
                autocommit=True)
            yield cnxn

        except Exception as e:
            print(e)

    val = 0

    try:
        with connection() as conn:
            curr = conn.cursor()
            params = (package_name, new_package_name)

            CreateProjectCommand = "SET NOCOUNT ON EXEC [elt].[Clone Foreach Data Flow Package] ?, ?"
            curr.execute(CreateProjectCommand, params, )
            val = curr.fetchall()[0][0]
    except pyodbc.Error as e:
        print(e)

    if val > 0:
        print(f"Succesfully cloned project with qualifier {new_package_name} !")
        create_html(server_name, database_name, new_package_name, command_type="foreach_data_flow")


def create_execute_process_package(server_name, database_name, package_qualifier, executable_expression,
                             arguments_expession, working_directory, place_values_in_ELT_Data, command="regular"):

    if place_values_in_ELT_Data.strip() == '' or place_values_in_ELT_Data == 'place values in ELT_Data default(0)':
        place_values_in_ELT_Data = 0



    @contextmanager
    def connection():
        try:
            cnxn = pyodbc.connect(
                "Driver={SQL Server Native Client 11.0};Server=" + server_name + ";Database=" + database_name + ";Trusted_Connection=yes;MARS_Connection=Yes;",
                autocommit=True)
            yield cnxn

        except Exception as e:
            print(e)

    val = 0

    try:
        with connection() as conn:
            curr = conn.cursor()
            params = (package_qualifier, executable_expression,
                       arguments_expession, working_directory, place_values_in_ELT_Data)

            CreateProjectCommand = "SET NOCOUNT ON EXEC [elt].[Save Execute Process Package] ?, ?, ?, ?, ?"
            curr.execute(CreateProjectCommand, params, )
            val = curr.fetchall()[0][0]
    except pyodbc.Error as e:
        print("Something got wrong")
        return

    if val:
        p_name = 'Execute Process ' + package_qualifier
        print(f"Succesfully created Execute Process pacakge {p_name}!")
        create_html(server_name, database_name, p_name, command_type="execute_process")


def clone_execute_process_package(server_name, database_name, package_name, new_package_name):
    @contextmanager
    def connection():
        try:
            cnxn = pyodbc.connect(
                "Driver={SQL Server Native Client 11.0};Server=" + server_name + ";Database=" + database_name + ";Trusted_Connection=yes;MARS_Connection=Yes;",
                autocommit=True)
            yield cnxn

        except Exception as e:
            print(e)

    val = 0

    try:
        with connection() as conn:
            curr = conn.cursor()
            params = (package_name, new_package_name)

            CreateProjectCommand = "SET NOCOUNT ON EXEC [elt].[Clone Execute Process Package v2] ?, ?"
            curr.execute(CreateProjectCommand, params, )
            val = curr.fetchall()[0][0]
    except pyodbc.Error as e:
        print(e)

    if val > 0:
        print(f"Succesfully cloned project with qualifier {new_package_name} !")
        create_html(server_name, database_name, new_package_name, command_type="execute_process")

def create_execute_SQL_package(server_name, database_name, package_qualifier, connection_name_source,
                             query, is_expression, return_row_count):

    if is_expression.strip() == '' or is_expression == 'is_expression(default 0)':
        is_expression = 0

    if return_row_count.strip() == '' or is_expression == 'return_row_count(default 0)':
        return_row_count = 0

    @contextmanager
    def connection():
        try:
            cnxn = pyodbc.connect(
                "Driver={SQL Server Native Client 11.0};Server=" + server_name + ";Database=" + database_name + ";Trusted_Connection=yes;MARS_Connection=Yes;",
                autocommit=True)
            yield cnxn

        except Exception as e:
            print(e)
    val = 0

    try:
        with connection() as conn:
            curr = conn.cursor()
            params = (package_qualifier, connection_name_source,
                      query, is_expression, return_row_count)

            CreateProjectCommand = "SET NOCOUNT ON EXEC [elt].[Save Execute SQL Package] ?, ?, ?, ?, ?"
            curr.execute(CreateProjectCommand, params, )
            val = curr.fetchall()[0][0]
    except pyodbc.Error as e:
        print("Something got wrong")
        return

    if val:
        p_name = 'Execute SQL ' + package_qualifier
        print(f"Succesfully created Execute SQL pacakge {p_name}!")
        create_html(server_name, database_name, p_name, command_type="execute_sql")

def clone_execute_SQL_package(server_name, database_name, package_name, new_package_qualifier):

    @contextmanager
    def connection():
        try:
            cnxn = pyodbc.connect(
                "Driver={SQL Server Native Client 11.0};Server=" + server_name + ";Database=" + database_name + ";Trusted_Connection=yes;MARS_Connection=Yes;",
                autocommit=True)
            yield cnxn

        except Exception as e:
            print(e)
    val = 0

    try:
        with connection() as conn:
            curr = conn.cursor()
            params = (package_name, new_package_qualifier)

            CreateProjectCommand = "SET NOCOUNT ON EXEC [elt].[Clone Execute SQL Package v2] ?, ?"
            curr.execute(CreateProjectCommand, params, )
            val = curr.fetchall()[0][0]
    except pyodbc.Error as e:
        print(e)

    if val > 0:
        print(f"Succesfully cloned project with qualifier {new_package_qualifier} !")
        create_html(server_name, database_name, new_package_qualifier, command_type="foreach_data_flow")


def create_foreach_execute_sql_package(server_name, database_name, foreach_connection, connection_name_source,
                               foreach_query, query, package_qualifier, is_expression, return_row_count, command="regular"):

    if is_expression.strip() == '' or is_expression == 'is_expression(default 0)':
        is_expression = 0

    if return_row_count.strip() == '' or is_expression == 'return_row_count(default 0)':
        return_row_count = 0

    @contextmanager
    def connection():
        try:
            cnxn = pyodbc.connect(
                "Driver={SQL Server Native Client 11.0};Server=" + server_name + ";Database=" + database_name + ";Trusted_Connection=yes;MARS_Connection=Yes;",
                autocommit=True)
            yield cnxn

        except Exception as e:
            print(e)

    val = 0

    try:
        with connection() as conn:
            curr = conn.cursor()
            params = (foreach_connection, connection_name_source,
                    foreach_query, query, package_qualifier, is_expression, return_row_count)

            CreateProjectCommand = "SET NOCOUNT ON EXEC [elt].[Save Foreach Execute SQL Package] ?, ?, ?, ?, ?, ?, ?"
            curr.execute(CreateProjectCommand, params, )
            val = curr.fetchall()[0][0]
    except pyodbc.Error as e:
        print("Something got wrong")
        return

    if val:
        p_name = 'Foreach Execute SQL ' + package_qualifier
        if command == "regular":
            print(f"Succesfully created foreach execute SQL pacakge {p_name}!")
        elif command == "clone":
            print(f"Succesfully cloned foreach execute SQL pacakge {p_name}!")
        create_html(server_name, database_name, p_name, command_type="foreach_execute_sql")


def clone_foreach_execute_SQL_package(server_name, database_name, package_name, new_package_qualifier):

    @contextmanager
    def connection():
        try:
            cnxn = pyodbc.connect(
                "Driver={SQL Server Native Client 11.0};Server=" + server_name + ";Database=" + database_name + ";Trusted_Connection=yes;MARS_Connection=Yes;",
                autocommit=True)
            yield cnxn

        except Exception as e:
            print(e)

    val = 0
    try:
        with connection() as conn:
            curr = conn.cursor()
            params = (package_name, new_package_qualifier)

            CreateProjectCommand = "SET NOCOUNT ON EXEC [elt].[Clone Foreach Execute SQL Package] ?, ?"
            curr.execute(CreateProjectCommand, params, )
            val = curr.fetchall()[0][0]
    except pyodbc.Error as e:
        print(e)

    if val > 0:
        print(f"Succesfully cloned project with qualifier {new_package_qualifier} !")
        create_html(server_name, database_name, new_package_qualifier, command_type="foreach_execute_sql")



#!/usr/bin/python
# -*- coding: <utf-8> -*-
import pyodbc
from contextlib import contextmanager
from pprint import pprint
import os
import webbrowser
from jinja2 import Environment,FileSystemLoader
import time
import shutil


def render(server_name, database_name, PATH):
    @contextmanager
    def connection():
        try:
            cnxn = pyodbc.connect(
                "Driver={SQL Server Native Client 11.0};Server=" + server_name + ";Database=" + database_name + ";Trusted_Connection=yes;MARS_Connection=Yes;")
            yield cnxn

        except Exception as e:
            print(e)

    with connection() as conn:
        curr = conn.cursor()
        get_projects = "SELECT TOP(1000)[project_name] FROM[eltsnap_v2].[elt].[project]"
        curr.execute(get_projects)
        c = curr.fetchall()
        project_set = [proj[0] for proj in c]

    from collections import defaultdict

    project_package_set = defaultdict(list)
    # projects with packages
    with connection() as conn:
        curr = conn.cursor()
        for i in project_set:
            get_project_package = "select p.project_name, pp.sequence_number, pp.package_name from elt.project as p right join [elt].[project_package] as pp on p.project_id=pp.project_id where p.project_name='{0}' order by  pp.sequence_number;".format(
                i)
            curr.execute(get_project_package)
            c = curr.fetchall()
            project_package_set[i].append(c)
            project_package_columns = [col[0] for col in curr.description][1:]

    with connection() as conn:
        # DF ppackage
        curr = conn.cursor()
        DataFlowPackage_ = "SELECT [src_connection],[src_query],[is_expression],[dst_connection] ,[dst_schema] ,[dst_table] ,[dst_truncate] ,[keep_identity] ,[package_name],[use_bulk_copy], [batch_size]" \
                           "  FROM [eltsnap_v2].[elt].[package_config_data_flow]"
        curr.execute(DataFlowPackage_)
        c = curr.fetchall()
        DF_table = c

    with connection() as conn:
        # FEDF package

        curr = conn.cursor()
        ForEachDF_ = "SELECT [foreach_connection] ,[foreach_query_expr],[src_connection],[src_query_expr],[dst_connection],[dst_schema],[dst_table],[dst_truncate],[keep_identity],[package_name]" \
                     ",[use_bulk_copy],[batch_size] FROM [eltsnap_v2].[elt].[package_config_foreach_data_flow]"
        curr.execute(ForEachDF_)
        c = curr.fetchall()
        ForEachDF = c

    with connection() as conn:
        # EXSQL package

        curr = conn.cursor()
        ExecSql_ = "SELECT [connection_manager],[query] ,[is_expression]   ,[return_row_count] ,[package_name] FROM [elt].[package_config_execute_sql]"
        curr.execute(ExecSql_)
        c = curr.fetchall()
        ExecSql = c

    with connection() as conn:
        # EXProc package

        curr = conn.cursor()
        ExecProc_ = "SELECT [executable_expr],[arguments_expr],[working_directory],[place_values_in_ELT_Data],[package_name] FROM [eltsnap_v2].[elt].[package_config_execute_process]"
        curr.execute(ExecProc_)
        c = curr.fetchall()
        ExecProc = c

    with connection() as conn:
        # JSONTT package

        curr = conn.cursor()
        JSONtt_ = "SELECT [src_connection] ,[table_selection_option],[table_list],[flat_file_connection],[dst_connection],[package_name] FROM [eltsnap_v2].[elt].[package_config_json_table_transfer]"
        curr.execute(JSONtt_)
        c = curr.fetchall()
        JSONtt = c

    with connection() as conn:
        # SS data package

        curr = conn.cursor()
        SemiS_ = "SELECT [src_connection],[dst_connection],[dst_schema],[dst_tables_init_option],[package_name] FROM [eltsnap_v2].[elt].[package_config_semi_struct_load]"
        curr.execute(SemiS_)
        c = curr.fetchall()
        SemiS = c

    with connection() as conn:
        # FEXSQL package
        curr = conn.cursor()
        ForEachSQL_ = "SELECT [foreach_connection],[foreach_query_expr],[query_connection],[query],[return_row_count],[package_name] FROM [eltsnap_v2].[elt].[package_config_foreach_execute_sql]"
        curr.execute(ForEachSQL_)
        c = curr.fetchall()
        ForEachSQL = c

    # project_enviroments
    enviroment_project = defaultdict(list)
    with connection() as conn:
        curr = conn.cursor()
        enviroment_project_st = "  select p.project_name, e.[environment_name] from [elt].[project] as p inner join [elt].[project_environment] as e on p.project_id=e.project_id;"
        curr.execute(enviroment_project_st)
        c = curr.fetchall()
        for e in c:
            enviroment_project[e[0]].append(e[1])
    # print(env_side)
    # exit()
    # pprint(enviroment_project)
    # exit()
    # for i,j in enviroment_project.items():
    #     # print(i,j)
    #     for m in j:
    #         print(m)
    # exit()

    # enviroments data details
    enviroments = defaultdict(list)
    with connection() as conn:
        curr = conn.cursor()
        env_side_ = "SELECT [environment_name],[build_template_group] FROM [elt].[environment]"
        curr.execute(env_side_)
        envs_build = curr.fetchall()
        env_side = [c[0] for c in envs_build]
        for i in env_side:
            enviroments_ = "SELECT ep.[parameter_name], e.[build_template_group], ep.[key_vault_name] FROM [elt].[environment_parameter] as ep inner join [elt].[environment] as e on ep.environment_name=e.environment_name where ep.environment_name='{0}'".format(
                i)
            curr.execute(enviroments_)
            c = curr.fetchall()
            enviroments[i].append(c)
            environ_package_columns = [col[0] for i, col in enumerate(curr.description) if i != 1]

    # projects_per enviroment
    proj_per_env = defaultdict(list)
    with connection() as conn:
        curr = conn.cursor()
        env_s_ = "SELECT distinct [environment_name] FROM [elt].[environment]"
        curr.execute(env_s_)
        ens = curr.fetchall()
        en = [c[0] for c in envs_build]
        for i in en:

            proj_per_env_query = "SELECT  p.[project_name] from elt.project as p right join" \
                                 " elt.project_environment as pe on pe.project_id=p.project_id where pe.environment_name = {0!r}".format(
                i)
            curr.execute(proj_per_env_query)
            c = curr.fetchall()
            c = [c[0] for c in c]
            for pair in c:
                proj_per_env[i].append(pair)

    # projects per pakackage
    proj_per_package_DF = defaultdict(list)
    with connection() as conn:
        curr = conn.cursor()
        projs_per_package_DF_ = "SELECT pp.package_name,p.project_name  FROM .[elt].[project_package] as pp  left join elt.project as p	on p.project_id=pp.project_id   where pp.package_name like '%Data Flow%';"
        curr.execute(projs_per_package_DF_)
        ppp = curr.fetchall()
        for pair in ppp:
            if pair[1] != None:
                proj_per_package_DF[pair[0]].append(pair[1])

    proj_per_package_FEDF = defaultdict(list)
    with connection() as conn:
        curr = conn.cursor()
        projs_per_package_FEDF_ = "SELECT pp.package_name, p.project_name  FROM .[elt].[project_package] as pp  left join elt.project as p	on p.project_id=pp.project_id   where pp.package_name like '%Foreach Data%';"
        curr.execute(projs_per_package_FEDF_)
        ppp = curr.fetchall()
        for pair in ppp:
            if pair[1] != None:
                proj_per_package_FEDF[pair[0]].append(pair[1])

    proj_per_package_EXSQL = defaultdict(list)
    with connection() as conn:
        curr = conn.cursor()
        projs_per_package_EXSQL_ = "SELECT pp.package_name, p.project_name  FROM .[elt].[project_package] as pp  left join elt.project as p	on p.project_id=pp.project_id   where pp.package_name like '%Execute SQL%';"
        curr.execute(projs_per_package_EXSQL_)
        ppp = curr.fetchall()
        for pair in ppp:
            if pair[1] != None:
                proj_per_package_EXSQL[pair[0]].append(pair[1])

    proj_per_package_FEEXSQL = defaultdict(list)
    with connection() as conn:
        curr = conn.cursor()
        projs_per_package_FEEXSQL_ = "SELECT pp.package_name, p.project_name FROM .[elt].[project_package] as pp left join elt.project as p on p.project_id=pp.project_id where pp.package_name like '%Foreach Execute SQL%';"
        curr.execute(projs_per_package_FEEXSQL_)
        ppp = curr.fetchall()
        for pair in ppp:
            if pair[1] != None:
                proj_per_package_FEEXSQL[pair[0]].append(pair[1])

    proj_per_package_EXPro = defaultdict(list)
    with connection() as conn:
        curr = conn.cursor()
        projs_per_package_EXPro_ = "SELECT pp.package_name, p.project_name  FROM .[elt].[project_package] as pp  left join elt.project as p	on p.project_id=pp.project_id   where pp.package_name like '%Execute Process%';"
        curr.execute(projs_per_package_EXPro_)
        ppp = curr.fetchall()
        for pair in ppp:
            if pair[1] != None:
                proj_per_package_EXPro[pair[0]].append(pair[1])

    proj_per_package_JJTS = defaultdict(list)
    with connection() as conn:
        curr = conn.cursor()
        projs_per_package_JJTS_ = "SELECT pp.package_name, p.project_name  FROM .[elt].[project_package] as pp  left join elt.project as p	on p.project_id=pp.project_id   where pp.package_name like '%Json%';"
        curr.execute(projs_per_package_JJTS_)
        ppp = curr.fetchall()
        for pair in ppp:
            if pair[1] != None:
                proj_per_package_JJTS[pair[0]].append(pair[1])

    proj_per_package_SSDT = defaultdict(list)
    with connection() as conn:
        curr = conn.cursor()
        projs_per_package_SSDT_ = "SELECT pp.package_name, p.project_name  FROM .[elt].[project_package] as pp  left join elt.project as p	on p.project_id=pp.project_id   where pp.package_name like '%Semi-Structured%';"
        curr.execute(projs_per_package_SSDT_)
        ppp = curr.fetchall()
        for pair in ppp:
            if pair[1] != None:
                proj_per_package_SSDT[pair[0]].append(pair[1])

    with connection() as conn:
        curr = conn.cursor()
        connections_query = "SELECT  [connection_name] FROM [eltsnap_v2].[elt].[oledb_connection]"
        curr.execute(connections_query)
        c = curr.fetchall()
        connections = [m[0] for m in c]

    with connection() as conn:
        curr = conn.cursor()
        connection_details_ = "SELECT [connection_name],[server_name],[database_name],[provider],[connection_expression] FROM [eltsnap_v2].[elt].[oledb_connection]"
        curr.execute(connection_details_)
        c = curr.fetchall()
        connection_details = c

    conns_projects = defaultdict(list)
    with connection() as conn:
        curr = conn.cursor()
        get_conections_by_projects = "SELECT *FROM [elt].[show projects using connection] (?)"
        for k in connections:
            curr.execute(get_conections_by_projects, k)
            c = curr.fetchall()
            c1 = [p[0] for p in c]
            conns_projects[k].append(c1)

    conns_packages = defaultdict(list)
    with connection() as conn:
        curr = conn.cursor()
        get_conections_by_packages = "SELECT * FROM [elt].[show packages using connection] (?)"
        for k in connections:
            curr.execute(get_conections_by_packages, k)
            c = curr.fetchall()
            c1 = [p[0] for p in c]
            conns_packages[k].append(c1)

    import datetime
    filename = os.path.join(PATH, 'eltSnap_Project_HTML.html')
    root = os.path.dirname(os.path.abspath(__file__))
    templates_dir = os.path.join(root, 'html')

    shutil.copy2(os.path.join(templates_dir, 'eltstyle.css'), PATH)
    shutil.copy2(os.path.join(templates_dir, 'images', 'eltSnap_whiteTransparent.png'), PATH)

    env = Environment(loader=FileSystemLoader(templates_dir))
    template = env.get_template('HTML_template.html')

    with open(filename, 'w') as writer:
        writer.write(template.render(
            h1="eltSnap Projects",
            published=datetime.datetime.now().strftime('%m/%d/%Y, %H:%M:%S'),
            project_name="Project Name",
            project_set=project_set,
            project_package_set=project_package_set,
            project_package_columns=project_package_columns,
            proj_per_package_DF=proj_per_package_DF,
            proj_per_package_FEDF=proj_per_package_FEDF,
            proj_per_package_EXSQL=proj_per_package_EXSQL,
            proj_per_package_FEEXSQL=proj_per_package_FEEXSQL,
            proj_per_package_EXPro=proj_per_package_EXPro,
            proj_per_package_JJTS=proj_per_package_JJTS,
            proj_per_package_SSDT=proj_per_package_SSDT,
            DF_table=DF_table,
            ExecSql=ExecSql,
            ForEachDF=ForEachDF,
            ExecProc=ExecProc,
            ForEachSQL=ForEachSQL,
            connections=connections,
            connection_details=connection_details,
            conns_projects=conns_projects,
            conns_packages=conns_packages,
            enviroment_project=enviroment_project,
            enviroments=enviroments,
            environ_package_columns=environ_package_columns,
            env_side=env_side,
            envs_build=envs_build,
            proj_per_env=proj_per_env,
            JSONtt=JSONtt,
            SemiS=SemiS
        ))

    # time.sleep(0.1)
    print(f'The HTML file destination is on the location : {filename}')


import pyodbc
from contextlib import contextmanager
import os
from jinja2 import Environment, FileSystemLoader
import time
import datetime
from tabulate import tabulate
from .eltSnap_Project_HTML import render

def create_html(server_name, database_name, proj_name, command_type ="regular"):
    @contextmanager
    def connection():
        try:
            cnxn = pyodbc.connect(
                "Driver={SQL Server Native Client 11.0};Server=" + server_name + ";Database=" + database_name + ";Trusted_Connection=yes;MARS_Connection=Yes;",
                autocommit=True)
            yield cnxn

        except Exception as e:
            print(e)

    try:
        with connection() as conn:
            curr = conn.cursor()
            #Create new Project
            if command_type == "regular":
                new_project = f"SELECT TOP 20 [project_id],[project_name],[build_template_group] FROM [elt].project order by [project_name] "
            elif command_type == "connection":
                new_project = f"SELECT TOP (20) [connection_name],[server_name],[database_name],[provider],[custom_connect_string] FROM [elt].[oledb_connection] order by [connection_name] desc"
            elif command_type == "data_flow":
                new_project = f"SELECT TOP (20) * FROM [elt].[package_config_data_flow]"
            elif command_type == "foreach_data_flow":
                new_project = f"SELECT TOP (20) * FROM [elt].[package_config_foreach_data_flow]"
            elif command_type == "execute_process":
                new_project = f"SELECT TOP (20) * FROM [elt].[package_config_execute_process]"
            elif command_type == "execute_sql":
                new_project = f"SELECT TOP (20) * FROM [elt].[package_config_execute_sql]"
            elif command_type == "foreach_execute_sql":
                new_project = f"SELECT TOP (20) * FROM [elt].[package_config_foreach_execute_sql]"

            curr.execute(new_project)
            resultSet_ = curr.fetchall()
            columns = [col[0] for col in curr.description]
            project_parameters = resultSet_
    except Exception as e:
        print(e)

    # path = check_output(["where", "azuredatastudio"]).decode("utf-8")
    # path_to_azuredatastudio = path.split('\n')[0].strip('\r')
    # path_to_extension = os.path.join(path_to_azuredatastudio, '..', '..', 'resources', 'app', 'extensions', 'python')

    try:
        with connection() as conn:
            curr = conn.cursor()
            # Create new Project
            has_db_var = f"select setting from [elt].[application_config] where setting='path to html files location'"
            curr.execute(has_db_var)
            has_db_variable = curr.fetchall()
    except Exception as el:
        pass

    if not has_db_variable:
        with connection() as conn:
            curr = conn.cursor()
            insert_path = f"INSERT INTO [elt].[application_config] ([setting], [use_value]) VALUES ('path to html files location', '')"
            curr.execute(insert_path)

    try:
        with connection() as conn:
            curr = conn.cursor()
            new_project = f"SELECT [use_value] FROM [elt].[application_config] WHERE setting='path to html files location'"
            curr.execute(new_project)
            PATH = curr.fetchall()[0][0]
    except Exception as ex:
        pass

    try:
        PATH
    except NameError:
        PATH = ''


    while not os.path.exists(PATH):
        print(
            "There is no PATH to the directory in which eltSnap can build the html reports! Please provide absolute path for the directory")
        PATH = input()
        if not os.path.exists(PATH):
            print("The PATH does not exists")
        else:
            try:
                insert_path = f"UPDATE [elt].[application_config] SET use_value = '{PATH}' WHERE setting = 'path to html files location'"
                curr.execute(insert_path)
            except Exception as e:
                print(e)

    if os.path.exists(PATH):

        render(server_name, database_name, PATH)

    else:
        raise ("Path is wrong ! Check your azuredatastudio installaton !")


    time.sleep(0.1)
    print(tabulate(project_parameters, headers=columns))



import setuptools

setuptools.setup(
    name='eltsnap',
    author='Jim Miller',
    version='0.0.2',
    license='MIT license',
    description='Third version test',
    packages=['PyADSNewVersionAlpha'],
    package_dir={'PyADSNewVersionAlpha': 'PyADSNewVersionAlpha'},
    package_data={'PyADSNewVersionAlpha': ['html/*', 'html/bootstrap-4.3.1-dist/css/*',
                                           'html/bootstrap-4.3.1-dist/js/*',
                                           'html/images/*'], },
    include_package_data=True,
    install_requires=[
        'pyodbc',
        'jinja2',
        'tabulate'
    ],
    classifiers=[
        'Programming Language :: Python :: 3'
    ]
)



import pyodbc
from contextlib import contextmanager
from .create import create_html
import webbrowser

def open_webbrowser(filename):
    url = webbrowser.open(filename, new=2)

def create_project(server_name, database_name, GIVE_YOUR_PROJECT_A_NAME, SELECT_A_TEMPLATE_GROUP):
    if GIVE_YOUR_PROJECT_A_NAME == 'project name':
        raise ValueError("Can't name cloned project as new project name.")

    val = 0
    @contextmanager
    def connection():
        try:
            cnxn = pyodbc.connect(
                "Driver={SQL Server Native Client 11.0};Server=" + server_name + ";Database=" + database_name + ";Trusted_Connection=yes;MARS_Connection=Yes;",
                autocommit=True)
            yield cnxn

        except Exception as e:
            print(e)

    try:
        with connection() as conn:
            curr = conn.cursor()
            params = (GIVE_YOUR_PROJECT_A_NAME, SELECT_A_TEMPLATE_GROUP)
    
            CreateProjectCommand = "SET NOCOUNT ON EXEC [elt].[Save Project] @project_name =?, @template_group = ?"
            curr.execute(CreateProjectCommand, params,)
            val = curr.fetchall()[0][0]
    except pyodbc.Error as e:
        pass

    if val > 0:
        print(f"Succesfully created project with id {val} !")
        create_html(server_name, database_name, GIVE_YOUR_PROJECT_A_NAME)
    else:
        print(f"The project with such name already exists Try again!")  
        return


def create_data_connection(server_name, database_name, connection_name_to_save, server_name_to_save, database_name_to_save, provider, project_name, connection_string: str):
    if connection_string.strip() == '' or connection_string == 'custom connection string (optional)':
        connection_string = None

    if project_name == 'enter project name (optional)':
        project_name = ''

    @contextmanager
    def connection():
        try:
            cnxn = pyodbc.connect(
                "Driver={SQL Server Native Client 11.0};Server=" + server_name + ";Database=" + database_name + ";Trusted_Connection=yes;MARS_Connection=Yes;",
                autocommit=True)
            yield cnxn

        except Exception as e:
            print(e)

    val = 0

    try:
        with connection() as conn:
            curr = conn.cursor()
            params = (connection_name_to_save, server_name_to_save, database_name_to_save, provider, project_name, connection_string)

            CreateProjectCommand = "SET NOCOUNT ON EXEC [elt].[Save Connection by Name] ?, ?, ?, ?, ?, ?"
            curr.execute(CreateProjectCommand, params, )
            val = curr.fetchall()[0][0]
    except pyodbc.Error as e:
        print("Something got wrong")
        return

    if val:
        print(f"Succesfully created data connection {connection_name_to_save}!")
        create_html(server_name, database_name, connection_name_to_save, command_type="connection")

def clone_data_connection(server_name, database_name, old_connection_name, connection_name_to_save, project_name):
    if project_name is 'enter project name (optional)':
        project_name = ''

    @contextmanager
    def connection():
        try:
            cnxn = pyodbc.connect(
                "Driver={SQL Server Native Client 11.0};Server=" + server_name + ";Database=" + database_name + ";Trusted_Connection=yes;MARS_Connection=Yes;",
                autocommit=True)
            yield cnxn

        except Exception as e:
            print(e)

    val = 0

    try:
        with connection() as conn:
            curr = conn.cursor()
            params = (old_connection_name, connection_name_to_save, project_name)
            CreateProjectCommand = "SET NOCOUNT ON EXEC [elt].[Clone Connection By Name] ?, ?, ?"
            curr.execute(CreateProjectCommand, params, )
            val = curr.fetchall()[0][0]
    except pyodbc.Error as e:
        pass

    if val > 0:
        print(f"Succesfully cloned data connection with id {val} !")
        create_html(server_name, database_name, connection_name_to_save, command_type="connection")
    else:
        print(f"Error! Check your input parameters !")
        return

def delete_data_connection(server_name, database_name, connection_name_to_delete, project_name=''):
    @contextmanager
    def connection():
        try:
            cnxn = pyodbc.connect(
                "Driver={SQL Server Native Client 11.0};Server=" + server_name + ";Database=" + database_name + ";Trusted_Connection=yes;MARS_Connection=Yes;",
                autocommit=True)
            yield cnxn

        except Exception as e:
            print(e)

    val = 0

    try:
        with connection() as conn:
            curr = conn.cursor()
            connection_type = "Oledb"
            params = (connection_name_to_delete, connection_type, 0)

            CreateProjectCommand = "SET NOCOUNT ON EXEC [elt].[Delete Connection] ?, ?, ?"
            curr.execute(CreateProjectCommand, params, )
            val = curr.fetchall()[0][0]
    except pyodbc.Error as e:
        pass

    if val > 0:
        print(f"Succesfully deleted connection with name {connection_name_to_delete} !")
        create_html(server_name, database_name, connection_name_to_delete, command_type="connection")
    else:
        print(f"Something went wrong! Check for your inputs !")
        return

def delete_project(server_name, database_name, Select_Project):
    @contextmanager
    def connection():
        try:
            cnxn = pyodbc.connect(
                "Driver={SQL Server Native Client 11.0};Server=" + server_name + ";Database=" + database_name + ";Trusted_Connection=yes;MARS_Connection=Yes;", autocommit=True)
            yield cnxn
    
        except Exception as e:
            print(e)

    val = 0
    try:
        with connection() as conn:
            curr = conn.cursor()
            params = (Select_Project)
    
            CreateProjectCommand = "SET NOCOUNT ON EXEC [elt].[Delete Project] @project_name =?"
            curr.execute(CreateProjectCommand, params,)
            val = curr.fetchall()[0][0]
    except pyodbc.Error as e:
        pass


    if val >0:
        print(f"Succesfully deleted project with id {val} !")
        create_html(server_name, database_name, Select_Project)


    else:
        print(f"Error ! Check your input parameters!")
        return

def clone_project(server_name, database_name, Select_Project, New_Project_Name):
    if New_Project_Name == 'new project name':
        raise ValueError("Can't name cloned project as new project name.")

    @contextmanager
    def connection():
        try:
            cnxn = pyodbc.connect(
                "Driver={SQL Server Native Client 11.0};Server=" + server_name + ";Database=" + database_name + ";Trusted_Connection=yes;MARS_Connection=Yes;",autocommit=True)
            yield cnxn
        except Exception as ex:
            print(ex)

    val = None

    try:
        with connection() as conn:
            curr = conn.cursor()
            params = (New_Project_Name, Select_Project)
            CreateProjectCommand = "SET NOCOUNT ON EXEC [elt].[Clone Project By Name] @project_name=?, @base_project_name =?"
            curr.execute(CreateProjectCommand, params, )
            val = curr.fetchall()[0][0]
    except pyodbc.Error as e:
        pass

    if val is 0 or val is None:
        print(f"Error ! Check your input parameters!")
        return
    else:
        print(f"Cloned project {Select_Project} successfully to {New_Project_Name} !")
        create_html(server_name, database_name, New_Project_Name)

def rename_project(server_name, database_name, Select_Project, New_Project_Name):
    if New_Project_Name == 'new project name':
        raise ValueError("Can't name new project as 'new project name' !")
    @contextmanager
    def connection():
        try:
            cnxn = pyodbc.connect(
                "Driver={SQL Server Native Client 11.0};Server=" + server_name + ";Database=" + database_name + ";Trusted_Connection=yes;MARS_Connection=Yes;",
                autocommit=True)
            yield cnxn

        except Exception as e:
            print(e)
    val = 0
    try:
        with connection() as conn:
            curr = conn.cursor()
            CreateProjectCommand = f"SELECT [project_id],[build_template_group] FROM [elt].[project] where project_name= '{Select_Project}'"
            curr.execute(CreateProjectCommand)
            proje_id, tmp_group = curr.fetchall()[0]
    except pyodbc.Error as e:
        print(e)
    except NameError as ne:
        print("Such Project name does not exist inside our database.")

    try:
        with connection() as conn:
            curr = conn.cursor()
            params = (New_Project_Name, tmp_group, proje_id)

            CreateProjectCommand = "SET NOCOUNT ON EXEC [elt].[Save Project] @project_name =?, @template_group =?, @project_id =?"
            curr.execute(CreateProjectCommand, params, )
            val = curr.fetchall()[0][0]
    except pyodbc.Error as e:
        print(e)

    if val is None:
        print(f"Error ! Check your input parameters!")
        return
    else:
        print(f"Renamed project successfully to {New_Project_Name} !")
        create_html(server_name, database_name, New_Project_Name)


def change_project_template(server_name, database_name, Select_Project, New_Template_Group):
    @contextmanager
    def connection():
        try:
            cnxn = pyodbc.connect(
                "Driver={SQL Server Native Client 11.0};Server=" + server_name + ";Database=" + database_name + ";Trusted_Connection=yes;MARS_Connection=Yes;",
                autocommit=True)
            yield cnxn

        except Exception as e:
            print(e)

    val = 0
    try:
        with connection() as conn:
            curr = conn.cursor()
            CreateProjectCommand = f"SELECT [project_id] FROM [elt].[project] where project_name= '{Select_Project}'"
            curr.execute(CreateProjectCommand)
            proje_id = curr.fetchall()[0][0]
    except pyodbc.Error as e:
        pass
    except NameError as ne:
        print("Such Project name does not exist inside our database.")


    try:
        with connection() as conn:
            curr = conn.cursor()
            params = (Select_Project, New_Template_Group, proje_id)
            CreateProjectCommand = "SET NOCOUNT ON EXEC [elt].[Save Project] @project_name =?, @template_group =?, @project_id =?"
            curr.execute(CreateProjectCommand, params, )
            val = curr.fetchall()[0][0]
    except pyodbc.Error as e:
        pass

    if val is 0 or val is None:
        print(f"Error ! Check your input parameters!")
        return
    else:
        print(f"Renamed successfully projects  {Select_Project}  template group!")
        create_html(server_name, database_name, Select_Project)